package com.sun.corba.se.impl.presentation.rmi;

import java.lang.reflect.Method;
import java.rmi.UnexpectedException;
import org.omg.CORBA_2_3.portable.InputStream;
import org.omg.CORBA.portable.ApplicationException;
import org.omg.CORBA_2_3.portable.OutputStream;
import org.omg.CORBA.UserException;
import java.rmi.RemoteException;
import com.sun.corba.se.impl.logging.ORBUtilSystemException;

public class ExceptionHandlerImpl implements ExceptionHandler
{
    private ExceptionHandlerImpl.ExceptionRW[] rws;
    private final ORBUtilSystemException wrapper;
    
    public ExceptionHandlerImpl(Class[] array) {
        this.wrapper = ORBUtilSystemException.get("rpc.presentation");
        int n = 0;
        for (int i = 0; i < array.length; ++i) {
            if (!RemoteException.class.isAssignableFrom(array[i])) {
                ++n;
            }
        }
        this.rws = new ExceptionHandlerImpl.ExceptionRW[n];
        int n2 = 0;
        for (int j = 0; j < array.length; ++j) {
            Class<Object> clazz = array[j];
            if (!RemoteException.class.isAssignableFrom(clazz)) {
                ExceptionHandlerImpl.ExceptionRWIDLImpl exceptionRWIDLImpl;
                if (UserException.class.isAssignableFrom(clazz)) {
                    exceptionRWIDLImpl = new ExceptionHandlerImpl.ExceptionRWIDLImpl(clazz);
                }
                else {
                    exceptionRWIDLImpl = new ExceptionHandlerImpl.ExceptionRWRMIImpl(clazz);
                }
                this.rws[n2++] = exceptionRWIDLImpl;
            }
        }
    }
    
    private int findDeclaredException(Class clazz) {
        int i = 0;
        while (i < this.rws.length) {
            if (this.rws[i].getExceptionClass().isAssignableFrom(clazz)) {
                return i;
            }
            ++i;
        }
        return -1;
    }
    
    private int findDeclaredException(String s) {
        int i = 0;
        while (i < this.rws.length) {
            if (this.rws[i] == null) {
                return -1;
            }
            if (s.equals(this.rws[i].getId())) {
                return i;
            }
            ++i;
        }
        return -1;
    }
    
    @Override
    public boolean isDeclaredException(Class clazz) {
        return this.findDeclaredException(clazz) >= 0;
    }
    
    @Override
    public void writeException(OutputStream outputStream, Exception ex) {
        int declaredException = this.findDeclaredException(ex.getClass());
        if (declaredException < 0) {
            throw this.wrapper.writeUndeclaredException(ex, ex.getClass().getName());
        }
        this.rws[declaredException].write(outputStream, ex);
        return;
    }
    
    @Override
    public Exception readException(ApplicationException cause) {
        InputStream inputStream = (InputStream)cause.getInputStream();
        int declaredException = this.findDeclaredException(cause.getId());
        if (declaredException < 0) {
            UnexpectedException ex = new UnexpectedException(inputStream.read_string());
            ex.initCause(cause);
            return ex;
        }
        return this.rws[declaredException].read(inputStream);
    }
    
    public ExceptionHandlerImpl.ExceptionRW getRMIExceptionRW(Class clazz) {
        return new ExceptionHandlerImpl.ExceptionRWRMIImpl(clazz);
    }
    
    public abstract class ExceptionRWBase implements ExceptionHandlerImpl.ExceptionRW
    {
        private Class cls;
        private String id;
        
        public ExceptionRWBase(Class cls) {
            this.cls = cls;
        }
        
        @Override
        public Class getExceptionClass() {
            return this.cls;
        }
        
        @Override
        public String getId() {
            return this.id;
        }
        
        void setId(String id) {
            this.id = id;
        }
    }
    
    public class ExceptionRWIDLImpl extends ExceptionHandlerImpl.ExceptionRWBase
    {
        private Method readMethod;
        private Method writeMethod;
        
        public ExceptionRWIDLImpl(Class clazz) {
            super(clazz);
            String string = clazz.getName() + "Helper";
            ClassLoader classLoader = clazz.getClassLoader();
            Class<Object> forName;
            try {
                forName = Class.forName(string, true, classLoader);
                this.setId((String)forName.getDeclaredMethod("id", (Class[])null).invoke(null, (Object[])null));
            }
            catch (Exception ex) {
                throw ExceptionHandlerImpl.this.wrapper.badHelperIdMethod(ex, string);
            }
            try {
                this.writeMethod = forName.getDeclaredMethod("write", new Class[] { org.omg.CORBA.portable.OutputStream.class, clazz });
            }
            catch (Exception ex2) {
                throw ExceptionHandlerImpl.this.wrapper.badHelperWriteMethod(ex2, string);
            }
            try {
                this.readMethod = forName.getDeclaredMethod("read", new Class[] { org.omg.CORBA.portable.InputStream.class });
            }
            catch (Exception ex3) {
                throw ExceptionHandlerImpl.this.wrapper.badHelperReadMethod(ex3, string);
            }
        }
        
        @Override
        public void write(OutputStream outputStream, Exception ex) {
            try {
                this.writeMethod.invoke(null, new Object[] { outputStream, ex });
            }
            catch (Exception ex2) {
                throw ExceptionHandlerImpl.this.wrapper.badHelperWriteMethod(ex2, this.writeMethod.getDeclaringClass().getName());
            }
        }
        
        @Override
        public Exception read(InputStream inputStream) {
            try {
                return (Exception)this.readMethod.invoke(null, new Object[] { inputStream });
            }
            catch (Exception ex) {
                throw ExceptionHandlerImpl.this.wrapper.badHelperReadMethod(ex, this.readMethod.getDeclaringClass().getName());
            }
        }
    }
    
    public class ExceptionRWRMIImpl extends ExceptionHandlerImpl.ExceptionRWBase
    {
        public ExceptionRWRMIImpl(Class clazz) {
            super(clazz);
            this.setId(IDLNameTranslatorImpl.getExceptionId(clazz));
        }
        
        @Override
        public void write(OutputStream outputStream, Exception ex) {
            outputStream.write_string(this.getId());
            outputStream.write_value(ex, this.getExceptionClass());
        }
        
        @Override
        public Exception read(InputStream inputStream) {
            inputStream.read_string();
            return (Exception)inputStream.read_value(this.getExceptionClass());
        }
    }
    
    public interface ExceptionRW
    {
        public abstract Class getExceptionClass();
        
        public abstract String getId();
        
        public abstract void write(OutputStream p0, Exception p1);
        
        public abstract Exception read(InputStream p0);
    }
}
